@@ -5,6 +5,23 @@ jomo1ranmix.MCMCchain <-
     if (is.null(beta.start)) beta.start=matrix(0,ncol(X),(ncol(Y.con)+(sum(Y.numcat)-length(Y.numcat))))
     if (is.null(l1cov.start)) l1cov.start=diag(1,ncol(beta.start))
     if (is.null(l1cov.prior)) l1cov.prior=diag(1,ncol(beta.start))
+    if (is_tibble(Y.con)) {
+      Y.con<-data.frame(Y.con)
+      warning("tibbles not supported. Y.con converted to standard data.frame. ")
+    }
+    if (is_tibble(Y.cat)) {
+      Y.cat<-data.frame(Y.cat)
+      warning("tibbles not supported. Y.cat converted to standard data.frame. ")
+    }
+    if (is_tibble(X)) {
+      X<-data.frame(X)
+      warning("tibbles not supported. X converted to standard data.frame. ")
+    }
+    if (is_tibble(Z)) {
+      Z<-data.frame(Z)
+      warning("tibbles not supported. Z converted to standard data.frame. ")
+    }
+    
     clus<-factor(unlist(clus))
     previous_levels_clus<-levels(clus)
     levels(clus)<-0:(nlevels(clus)-1)
@@ -52,10 +69,39 @@ jomo1ranmix.MCMCchain <-
     storage.mode(Y.cat) <- "numeric"    
     X<-data.matrix(X)
     storage.mode(X) <- "numeric"    
+    stopifnot(!any(is.na(X)))
     Z<-data.matrix(Z)
-    storage.mode(Z) <- "numeric"    
+    storage.mode(Z) <- "numeric"
+    stopifnot(!any(is.na(Z)))
     clus <- matrix(as.integer(levels(clus))[clus], ncol=1)
     Y=cbind(Y.con,Y.cat)
+    if (any(is.na(Y))) {
+      if (ncol(Y)==1) {
+        miss.pat<-matrix(c(0,1),2,1)
+        n.patterns<-2
+      } else  {
+        miss.pat<-md.pattern.mice(Y, plot=F)
+        miss.pat<-miss.pat[,colnames(Y)]
+        n.patterns<-nrow(miss.pat)-1
+      }
+    } else {
+      miss.pat<-matrix(0,2,ncol(Y))
+      n.patterns<-nrow(miss.pat)-1
+    }
+    
+    miss.pat.id<-rep(0,nrow(Y))
+    for (i in 1:nrow(Y)) {
+      k <- 1
+      flag <- 0
+      while ((k <= n.patterns) & (flag == 0)) {
+        if (all(!is.na(Y[i,])==miss.pat[k,1:(ncol(miss.pat))])) {
+          miss.pat.id[i] <- k
+          flag <- 1
+        } else {
+          k <- k + 1
+        }
+      }
+    }
     Yi=cbind(Y.con, matrix(0,nrow(Y.con),(sum(Y.numcat)-length(Y.numcat))))
     h=1
     for (i in 1:length(Y.numcat)) {
@@ -102,7 +148,7 @@ jomo1ranmix.MCMCchain <-
     if (is.null(start.imp)) {
       for (i in 1:nrow(Yi)) for (j in 1:ncol(Yi)) if (is.na(Yimp[i,j])) Yimp2[i,j]=rnorm(1,meanobs[j],1)
     } 
-    .Call("jomo1ranC", Y, Yimp, Yimp2, Y.cat, X, Z, clus,betait,uit,betapost,upostall,covit,omegapost, covuit, covupost, nburn, l1cov.prior,l2cov.prior,Y.numcat, ncol(Y.con),out.iter, 1, PACKAGE = "jomo")
+    .Call("jomo1ranC", Y, Yimp, Yimp2, Y.cat, X, Z, clus,betait,uit,betapost,upostall,covit,omegapost, covuit, covupost, nburn, l1cov.prior,l2cov.prior,Y.numcat, ncol(Y.con),out.iter, 1, miss.pat.id, n.patterns, PACKAGE = "jomo")
     imp[(nrow(Y)+1):(2*nrow(Y)),1:ncol(Y.con)]=Yimp2[,1:ncol(Y.con)]
     imp[(nrow(Y)+1):(2*nrow(Y)),(ncol(Y.con)+1):ncol(Y)]=Y.cat
     
